package com.syntevo.plugin.jira.commit.messagesource;

import java.util.*;
import java.util.List;

import org.eclipse.jface.viewers.*;
import org.eclipse.swt.*;
import org.eclipse.swt.events.*;
import org.eclipse.swt.graphics.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.widgets.*;

import com.syntevo.openapi.gui.dialog.*;
import com.syntevo.plugin.jira.transport.*;

/**
 * @author syntevo GmbH
 */
final class JiraCommitMessageSourceDialog extends AbstractOkCancelDialog {

	// Fields =================================================================

	private final List<JiraIssue> initialIssues;
	private final JiraIssueLoader issueLoader;
	private final JiraQueryConfiguration queryConfiguration;

	private TableViewer tableViewer;
	private String commitMessage;

	// Setup ==================================================================

	public JiraCommitMessageSourceDialog(List<JiraIssue> initialIssues, JiraIssueLoader issueLoader, JiraQueryConfiguration queryConfiguration) {
		this.initialIssues = initialIssues;
		this.issueLoader = issueLoader;

		this.queryConfiguration = queryConfiguration;

		getOkAction().setName("Select");
	}

	// Implemented ============================================================

	@Override
	public String getTitle() {
		return "Select JIRA Issue";
	}

	@Override
	public Control createComponent(Composite parent) {
		final Composite panel = new Composite(parent, SWT.NONE) {
			@Override
			public Point computeSize(int wHint, int hHint, boolean changed) {
				return new Point(600, 400);
			}
		};
		panel.setLayout(new FillLayout());

		tableViewer = new TableViewer(new Table(panel, SWT.BORDER | SWT.FULL_SELECTION | SWT.H_SCROLL | SWT.V_SCROLL | SWT.MULTI));
		tableViewer.getTable().setHeaderVisible(true);
		tableViewer.setContentProvider(new IStructuredContentProvider() {
			@Override
			public Object[] getElements(Object inputElement) {
				final List<?> list = (List)inputElement;
				return list.toArray(new Object[list.size()]);
			}

			@Override
			public void dispose() {
			}

			@Override
			public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
			}
		});

		createColumn("Key", SWT.RIGHT, new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				return ((JiraIssue)element).getKey();
			}
		});

		createColumn("Summary", SWT.NONE, new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				return ((JiraIssue)element).getSummary();
			}
		});

		createColumn("Status", SWT.NONE, new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				return ((JiraIssue)element).getStatus().getName();
			}
		});

		tableViewer.getTable().addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
				performOk();
			}
		});

		setIssues(initialIssues);
		return panel;
	}

	@Override
	protected void ok() throws DialogValidationFailedException {
		@SuppressWarnings({"unchecked"})
		final List<JiraIssue> selectedIssues = ((IStructuredSelection)tableViewer.getSelection()).toList();
		if (selectedIssues.isEmpty()) {
			throw new DialogValidationFailedException(tableViewer.getTable(), "Please select one or more issue.", "Or press Cancel to abort.");
		}

		commitMessage = createCommitMessage(selectedIssues);
		super.ok();
	}

	@Override
	public Button initButtonPanel(ButtonPanel buttonPanel) {
		buttonPanel.addButton(new ButtonAction("Refresh", BUTTON_TYPE_OTHER) {
			@Override
			public void performAction() {
				issueLoader.load(queryConfiguration, true);
			}
		});
		return super.initButtonPanel(buttonPanel);
	}

	// Accessing ==============================================================

	public String getCommitMessage() {
		return commitMessage;
	}

	public void setIssues(final List<JiraIssue> issues) {
		final ISelection selection = tableViewer.getSelection();
		final int previousItemCount = tableViewer.getTable().getItemCount();
		tableViewer.setInput(issues);
		if (previousItemCount == 0) {
			for (TableColumn column : tableViewer.getTable().getColumns()) {
				column.pack();
			}
		}
		tableViewer.setSelection(selection);
	}

	// Utils ==================================================================

	private void createColumn(String name, int style, ColumnLabelProvider labelProvider) {
		final TableViewerColumn column = new TableViewerColumn(tableViewer, style);
		column.getColumn().setText(name);
		column.setLabelProvider(labelProvider);
		column.getColumn().pack();
	}

	private static String createCommitMessage(List<JiraIssue> issues) {
		final StringBuilder message = new StringBuilder();
		for (Iterator it = issues.iterator(); it.hasNext(); ) {
			final JiraIssue issue = (JiraIssue)it.next();
			message.append(issue.getKey());
			message.append(": ");
			message.append(issue.getSummary());
			if (it.hasNext()) {
				message.append("\n");
			}
		}

		return message.toString();
	}
}