package com.syntevo.plugin.trac.commit.messagesource;

import java.util.*;
import java.util.List;

import org.eclipse.jface.viewers.*;
import org.eclipse.swt.*;
import org.eclipse.swt.events.*;
import org.eclipse.swt.graphics.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.widgets.*;
import org.jetbrains.annotations.*;

import com.syntevo.openapi.gui.dialog.*;
import com.syntevo.openapi.smartsvn.command.commit.*;
import com.syntevo.plugin.trac.transport.*;

/**
 * @author syntevo GmbH
 */
final class TracCommitMessageSourceDialog extends AbstractOkCancelDialog {

	// Fields =================================================================

	private final List<TracIssue> initialIssues;
	private final TracIssueLoader issueLoader;
	private final TracQueryConfiguration queryConfiguration;
	private final DialogDisplayer dialogDisplayer;
	private final CommitBugtraqProperties bugtraqProperties;

	private TableViewer tableViewer;
	private String commitMessage;
	private String commitMessagePattern;

	// Setup ==================================================================

	public TracCommitMessageSourceDialog(@Nullable List<TracIssue> initialIssues, @NotNull TracIssueLoader issueLoader,
	                                     @Nullable TracQueryConfiguration queryConfiguration, @Nullable String commitMessagePattern,
	                                     @NotNull CommitBugtraqProperties bugtraqProperties, @NotNull DialogDisplayer dialogDisplayer) {
		this.initialIssues = initialIssues;
		this.issueLoader = issueLoader;
		this.queryConfiguration = queryConfiguration;
		this.commitMessagePattern = commitMessagePattern;
		this.bugtraqProperties = bugtraqProperties;
		this.dialogDisplayer = dialogDisplayer;

		getOkAction().setName("Select");
	}

	// Implemented ============================================================

	@NotNull
	@Override
	public String getTitle() {
		return "Select Trac Issue";
	}

	@NotNull
	@Override
	public Control createComponent(@NotNull Composite parent) {
		final Composite panel = new Composite(parent, SWT.NONE) {
			@Override
			public Point computeSize(int wHint, int hHint, boolean changed) {
				return new Point(600, 400);
			}
		};

		panel.setLayout(new FillLayout());

		tableViewer = new TableViewer(new Table(panel, SWT.BORDER | SWT.FULL_SELECTION | SWT.H_SCROLL | SWT.V_SCROLL | SWT.MULTI));
		tableViewer.getTable().setHeaderVisible(true);

		tableViewer.setContentProvider(new IStructuredContentProvider() {
			@Override
			public Object[] getElements(Object inputElement) {
				final List<?> list = (List)inputElement;
				return list.toArray(new Object[list.size()]);
			}

			@Override
			public void dispose() {
			}

			@Override
			public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
			}
		});

		createColumn("Id", SWT.RIGHT, new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				return "" + ((TracIssue)element).getId();
			}
		});

		createColumn("Summary", SWT.NONE, new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				return ((TracIssue)element).getSummary();
			}
		});

		createColumn("Status", SWT.NONE, new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				return ((TracIssue)element).getStatus().getName();
			}
		});

		createColumn("Milestone", SWT.NONE, new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				return ((TracIssue)element).getMilestoneName();
			}
		});

		createColumn("Version", SWT.NONE, new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				return ((TracIssue)element).getVersionName();
			}
		});

		tableViewer.getTable().addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
				performOk();
			}
		});

		setIssues(initialIssues);
		return panel;
	}

	@Override
	protected void ok() throws DialogValidationFailedException {
		@SuppressWarnings({"unchecked"})
		final List<TracIssue> selectedIssues = ((IStructuredSelection)tableViewer.getSelection()).toList();
		if (selectedIssues.isEmpty()) {
			throw new DialogValidationFailedException(tableViewer.getTable(), "Please select one or more issue.", "Or press Cancel to abort.");
		}

		if (commitMessagePattern != null) {
			commitMessagePattern = commitMessagePattern.trim();
		}

		commitMessage = createCommitMessage(selectedIssues, commitMessagePattern, bugtraqProperties);
		super.ok();
	}

	@NotNull
	@Override
	public Button initButtonPanel(@NotNull ButtonPanel buttonPanel) {
		buttonPanel.addButton(new ButtonAction("Refresh", BUTTON_TYPE_OTHER) {
			@Override
			public void performAction() {
				issueLoader.load(queryConfiguration, true);
			}
		});

		if (bugtraqProperties.getMessage() == null) {
			buttonPanel.addButton(new ButtonAction("Configure", BUTTON_TYPE_OTHER) {
				@Override
				public void performAction() {
					configureMessagePattern();
				}
			});
		}

		return super.initButtonPanel(buttonPanel);
	}

	// Accessing ==============================================================

	@NotNull
	public String getCommitMessage() {
		return commitMessage;
	}

	@Nullable
	public String getCommitMessagePattern() {
		return commitMessagePattern;
	}

	public void setIssues(@Nullable List<TracIssue> issues) {
		final ISelection selection = tableViewer.getSelection();
		final int previousItemCount = tableViewer.getTable().getItemCount();
		tableViewer.setInput(issues);

		if (previousItemCount == 0) {
			for (TableColumn column : tableViewer.getTable().getColumns()) {
				column.pack();
			}
		}

		tableViewer.setSelection(selection);
	}

	// Utils ==================================================================

	private void configureMessagePattern() {
		final TracConfigureCommitMessagePatternDialog messagePatternDialog = new TracConfigureCommitMessagePatternDialog(commitMessagePattern);
		dialogDisplayer.showAsync(messagePatternDialog, new IDialogResultHandler() {
			@Override
			public void handleDialogResult(int value) {
				if (value != AbstractDialog.RESULT_OK) {
					return;
				}

				commitMessagePattern = messagePatternDialog.getCommitMessagePattern();
			}
		});
	}

	private void createColumn(@NotNull String name, int style, @NotNull ColumnLabelProvider labelProvider) {
		final TableViewerColumn column = new TableViewerColumn(tableViewer, style);

		column.getColumn().setText(name);
		column.setLabelProvider(labelProvider);
		column.getColumn().pack();
	}

	@NotNull
	private static String createCommitMessage(@NotNull List<TracIssue> issues, @Nullable String commitMessagePattern, @NotNull CommitBugtraqProperties bugtraqProperties) {
		final StringBuilder idBuilder = new StringBuilder();
		final StringBuilder messageBuilder = new StringBuilder();
		final String bugtraqMessage = bugtraqProperties.getMessage();

		if (bugtraqMessage != null) {
			for (Iterator it = issues.iterator(); it.hasNext(); ) {
				final TracIssue issue = (TracIssue)it.next();

				idBuilder.append(bugtraqMessage.replace(TracConfigureCommitMessagePatternDialog.PLACEHOLDER_BUGID, String.valueOf(issue.getId())));

				if (it.hasNext()) {
					idBuilder.append(",");
				}
			}
		}

		for (Iterator it = issues.iterator(); it.hasNext(); ) {
			final TracIssue issue = (TracIssue)it.next();

			if (bugtraqMessage == null && commitMessagePattern != null && !commitMessagePattern.isEmpty()) {
				String message = commitMessagePattern.replace(TracConfigureCommitMessagePatternDialog.PLACEHOLDER_BUGID, String.valueOf(issue.getId()));
				message = message.replace(TracConfigureCommitMessagePatternDialog.PLACEHOLDER_MESSAGE, issue.getSummary());

				messageBuilder.append(message);
			}
			else {
				messageBuilder.append(issue.getSummary());
			}

			if (it.hasNext()) {
				messageBuilder.append("\n");
			}
		}

		if (bugtraqMessage != null) {
			if (bugtraqProperties.isAppend()) {
				return messageBuilder.toString() + "\n" + idBuilder.toString();
			}
			else {
				return idBuilder.toString() + "\n" + messageBuilder.toString();
			}
		}

		return messageBuilder.toString();
	}
}